/* getdiskfreespace.c -- get disk free space
   Copyright (C) 1999-2000 Wojciech Galazka
   Modified parts Copyright (C) 2016 Joe Forster/STA

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2, or (at your option)
   any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software Foundation,
   Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.  */

//
// Work around int 21h,AH=36h bug
//

#include "lfnsrv.h"

WORD	process_get_disk_free_space(
		IN  PCHAR name,
		OUT PDWORD dwSectorsPerCluster,
		OUT PDWORD dwBytesPerSector,
		OUT PDWORD dwFreeClusters,
		OUT PDWORD dwTotalClusters)
{
	WORD	result;

	PROLOG(process_get_disk_free_space);
	ISNULL(dwSectorsPerCluster);
	ISNULL(dwBytesPerSector   );
	ISNULL(dwFreeClusters     );
	ISNULL(dwTotalClusters    );
	DBGVALUE(name,"%s");


	if (!GetDiskFreeSpace(name, dwSectorsPerCluster, dwBytesPerSector,
			dwFreeClusters, dwTotalClusters)) {
		result = (WORD)GetLastError();
		DBGMSG("GetDiskFreeSpace call failed");
		DBGVALUE(result,"%d");
		if (result == ERROR_INVALID_DRIVE &&
		ValidDriveName(name, TRUE) == ERROR_SUCCESS) {
			*dwSectorsPerCluster = 0;
			*dwBytesPerSector    = 0;
			*dwFreeClusters      = 0;
			*dwTotalClusters     = 0;
		} else {
			EPILOG(process_get_disk_free_space, FALSE);
			return result;
		}
	}

	DBGVALUE(*dwSectorsPerCluster ,"%lx");
	DBGVALUE(*dwBytesPerSector    ,"%lx");
	DBGVALUE(*dwFreeClusters      ,"%lx");
	DBGVALUE(*dwTotalClusters     ,"%lx");

#if 0
	if (dwTotalClusters > 0xffff)  {
		char lpFsName[14];
		if (!GetVolumeInformation(name, NULL, 0, NULL,
				NULL, NULL, lpFsName, sizeof(lpFsName))) {
			result = (WORD)GetLastError();
			DBGVALUE(result,"%d");
			EPILOG(process_get_disk_free_space, FALSE);
			return result;
		}
		DBGVALUE(lpFsName,"%s");
		if (lstrcmp(lpFsName,"NTFS") == 0) {
			dwSectorsPerCluster <<= 8;
			dwFreeClusters 	    >>= 8;
			dwTotalClusters     >>= 8;
		}
	}

	*wSectorsPerCluster = (dwSectorsPerCluster >0xffff)? 0xffff:LOWORD(dwSectorsPerCluster);
	*wBytesPerSector    = (dwBytesPerSector    >0xffff)? 0xffff:LOWORD(dwBytesPerSector);
	*wFreeClusters      = (dwFreeClusters      >0xffff)? 0xffff:LOWORD(dwFreeClusters);
	*wTotalClusters     = (dwTotalClusters     >0xffff)? 0xffff:LOWORD(dwTotalClusters);
#else
// 	while (*dwTotalClusters >= (DWORD)0xffff) {
//		*dwSectorsPerCluster <<= 1;
//		*dwFreeClusters	     >>= 1;
//		*dwTotalClusters     >>= 1;
//	}

#endif

//	DBGVALUE(*dwSectorsPerCluster ,"%lx");
//	DBGVALUE(*dwBytesPerSector    ,"%lx");
//	DBGVALUE(*dwFreeClusters      ,"%lx");
//	DBGVALUE(*dwTotalClusters     ,"%lx");
	EPILOG(process_get_disk_free_space,TRUE);
	return ERROR_SUCCESS;
}
